-- Suppression des tables (si elles existent)
DROP TABLE IF EXISTS commentary;
DROP TABLE IF EXISTS favorite;
DROP TABLE IF EXISTS daily_quote;
DROP TABLE IF EXISTS quote;
DROP TABLE IF EXISTS caracter;
DROP TABLE IF EXISTS source;
DROP TABLE IF EXISTS record_quiz;
DROP TABLE IF EXISTS quiz_question;
DROP TABLE IF EXISTS quiz;
DROP TABLE IF EXISTS question;
DROP TABLE IF EXISTS admin;
DROP TABLE IF EXISTS users;
DROP TABLE IF EXISTS image;

-- Création des tables

-- Table image (l'id est auto-incrémenté)
CREATE TABLE image (
                       id_img INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
                       img_path VARCHAR(300) NOT NULL UNIQUE
);

-- Table users (l'id est auto-incrémenté)
CREATE TABLE users (
                       id_user INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
                       username VARCHAR(50) NOT NULL,
                       email VARCHAR(50) NOT NULL,
                       password VARCHAR(100) NOT NULL,
                       img INTEGER NOT NULL,
                       creation DATE NOT NULL,
                       CONSTRAINT unique_col UNIQUE (email),
                       CONSTRAINT fk_users_img FOREIGN KEY (img) REFERENCES image(id_img)
);

-- Table admin
-- La colonne 'users' doit contenir l'id d'un utilisateur existant (pas auto-incrémenté ici)
CREATE TABLE admin (
                       users INTEGER PRIMARY KEY,
                       CONSTRAINT fk_admin_user FOREIGN KEY (users) REFERENCES users(id_user)
);

-- Table question (l'id est auto-incrémenté)
CREATE TABLE question (
                          id_question INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
                          texte TEXT NOT NULL UNIQUE,
                          answer_a VARCHAR(30) NOT NULL,
                          answer_b VARCHAR(30) NOT NULL,
                          answer_c VARCHAR(30) NOT NULL,
                          answer_d VARCHAR(30) NOT NULL,
                          c_answer VARCHAR(30) NOT NULL,
                          CONSTRAINT check_c_answer CHECK (c_answer = answer_a OR c_answer = answer_b OR c_answer = answer_c OR c_answer = answer_d)
);

-- Table quiz (l'id est auto-incrémenté)
CREATE TABLE quiz (
                      id_quiz INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
                      title VARCHAR(40) NOT NULL,
                      img INTEGER NOT NULL,
                      nb_quest NUMERIC DEFAULT 0,
                      CONSTRAINT fk_quiz_img FOREIGN KEY (img) REFERENCES image(id_img)
);

-- Table quiz_question
-- Les colonnes ici sont des clés étrangères et doivent être de type INTEGER (les valeurs proviennent des tables quiz et question)
CREATE TABLE quiz_question (
                               quiz INTEGER NOT NULL,
                               question INTEGER NOT NULL,
                               PRIMARY KEY (quiz, question),
                               CONSTRAINT fk_quiz_question_quiz FOREIGN KEY (quiz) REFERENCES quiz(id_quiz),
                               CONSTRAINT fk_quiz_question_question FOREIGN KEY (question) REFERENCES question(id_question)
);

-- Table record_quiz
CREATE TABLE record_quiz (
                             users INTEGER NOT NULL,
                             quiz INTEGER NOT NULL,
                             nb_point NUMERIC DEFAULT 0,
                             time_q NUMERIC DEFAULT 0,
                             PRIMARY KEY (users, quiz),
                             CONSTRAINT fk_recordquiz_user FOREIGN KEY (users) REFERENCES users(id_user),
                             CONSTRAINT fk_recordquiz_quiz FOREIGN KEY (quiz) REFERENCES quiz(id_quiz),
                             CONSTRAINT err_nb_point CHECK (nb_point >= 0),
                             CONSTRAINT err_time_q CHECK (time_q >= 0)
);

-- Table source (l'id est auto-incrémenté)
CREATE TABLE source (
                        id_source INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
                        title VARCHAR(100) NOT NULL,
                        date_s NUMERIC(4) NOT NULL
);

-- Table caracter (l'id est auto-incrémenté)
CREATE TABLE caracter (
                          id_caracter INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
                          caracter VARCHAR(100) NOT NULL,
                          id_img INTEGER NOT NULL,
                          CONSTRAINT fk_caracter_image FOREIGN KEY (id_img) REFERENCES image(id_img)
);

-- Table quote (l'id est auto-incrémenté)
CREATE TABLE quote (
                       id_quote INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
                       content TEXT NOT NULL,
                       likes NUMERIC DEFAULT 0,
                       langue CHAR(2) NOT NULL,
                       is_valide BOOLEAN NOT NULL DEFAULT FALSE,
                       reason VARCHAR(100) NOT NULL,
                       id_caracter INTEGER NOT NULL,
                       id_source INTEGER NOT NULL,
                       id_user_verif INTEGER NOT NULL,
                       CONSTRAINT fk_quote_caracter FOREIGN KEY (id_caracter) REFERENCES caracter(id_caracter),
                       CONSTRAINT fk_quote_source FOREIGN KEY (id_source) REFERENCES source(id_source),
                       CONSTRAINT fk_quote_userverif FOREIGN KEY (id_user_verif) REFERENCES users(id_user),
                       CONSTRAINT err_nb_like CHECK (likes >= 0),
                       CONSTRAINT err_language CHECK (langue = 'fr' OR langue = 'en')
);

-- Table daily_quote
CREATE TABLE daily_quote (
                             citation_id INTEGER PRIMARY KEY,
                             FOREIGN KEY (citation_id) REFERENCES quote(id_quote) ON DELETE CASCADE
);

-- Table favorite
CREATE TABLE favorite (
                          users INTEGER NOT NULL,
                          quote INTEGER NOT NULL,
                          PRIMARY KEY (users, quote),
                          CONSTRAINT fk_favorite_quote FOREIGN KEY (quote) REFERENCES quote(id_quote),
                          CONSTRAINT fk_favorite_user FOREIGN KEY (users) REFERENCES users(id_user)
);

-- Table commentary (l'id est auto-incrémenté)
CREATE TABLE commentary (
                            id_comment INTEGER GENERATED BY DEFAULT AS IDENTITY PRIMARY KEY,
                            quote INTEGER NOT NULL,
                            users INTEGER NOT NULL,
                            date_c DATE NOT NULL,
                            comment TEXT NOT NULL,
                            CONSTRAINT fk_commentary_quote FOREIGN KEY (quote) REFERENCES quote(id_quote),
                            CONSTRAINT fk_commentary_user FOREIGN KEY (users) REFERENCES users(id_user)
);
